# define Y_EOF 257
# define Y_NL 258
# define Y_INT 259
# define Y_ID 260
# define Y_REG 261
# define Y_FP_REG 262
# define Y_STR 263
# define Y_FP 264
# define Y_ABS_D_OP 265
# define Y_ABS_S_OP 266
# define Y_ADDIU_OP 267
# define Y_ADDI_OP 268
# define Y_ADDU_OP 269
# define Y_ADD_D_OP 270
# define Y_ADD_OP 271
# define Y_ADD_S_OP 272
# define Y_ANDI_OP 273
# define Y_AND_OP 274
# define Y_BC0F_OP 275
# define Y_BC0T_OP 276
# define Y_BC1F_OP 277
# define Y_BC1T_OP 278
# define Y_BC2F_OP 279
# define Y_BC2T_OP 280
# define Y_BC3F_OP 281
# define Y_BC3T_OP 282
# define Y_BEQ_OP 283
# define Y_BGEZAL_OP 284
# define Y_BGEZ_OP 285
# define Y_BGTZ_OP 286
# define Y_BLEZ_OP 287
# define Y_BLTZAL_OP 288
# define Y_BLTZ_OP 289
# define Y_BNE_OP 290
# define Y_BREAK_OP 291
# define Y_CFC0_OP 292
# define Y_CFC1_OP 293
# define Y_CFC2_OP 294
# define Y_CFC3_OP 295
# define Y_COP0_OP 296
# define Y_COP1_OP 297
# define Y_COP2_OP 298
# define Y_COP3_OP 299
# define Y_CTC0_OP 300
# define Y_CTC1_OP 301
# define Y_CTC2_OP 302
# define Y_CTC3_OP 303
# define Y_CVT_D_S_OP 304
# define Y_CVT_D_W_OP 305
# define Y_CVT_S_D_OP 306
# define Y_CVT_S_W_OP 307
# define Y_CVT_W_D_OP 308
# define Y_CVT_W_S_OP 309
# define Y_C_EQ_D_OP 310
# define Y_C_EQ_S_OP 311
# define Y_C_F_D_OP 312
# define Y_C_F_S_OP 313
# define Y_C_LE_D_OP 314
# define Y_C_LE_S_OP 315
# define Y_C_LT_D_OP 316
# define Y_C_LT_S_OP 317
# define Y_C_NGE_D_OP 318
# define Y_C_NGE_S_OP 319
# define Y_C_NGLE_D_OP 320
# define Y_C_NGLE_S_OP 321
# define Y_C_NGL_D_OP 322
# define Y_C_NGL_S_OP 323
# define Y_C_NGT_D_OP 324
# define Y_C_NGT_S_OP 325
# define Y_C_OLT_D_OP 326
# define Y_C_OLT_S_OP 327
# define Y_C_OLE_D_OP 328
# define Y_C_OLE_S_OP 329
# define Y_C_SEQ_D_OP 330
# define Y_C_SEQ_S_OP 331
# define Y_C_SF_D_OP 332
# define Y_C_SF_S_OP 333
# define Y_C_UEQ_D_OP 334
# define Y_C_UEQ_S_OP 335
# define Y_C_ULT_D_OP 336
# define Y_C_ULT_S_OP 337
# define Y_C_ULE_D_OP 338
# define Y_C_ULE_S_OP 339
# define Y_C_UN_D_OP 340
# define Y_C_UN_S_OP 341
# define Y_DIVU_OP 342
# define Y_DIV_D_OP 343
# define Y_DIV_OP 344
# define Y_DIV_S_OP 345
# define Y_JALR_OP 346
# define Y_JAL_OP 347
# define Y_JR_OP 348
# define Y_J_OP 349
# define Y_LBU_OP 350
# define Y_LB_OP 351
# define Y_LHU_OP 352
# define Y_LH_OP 353
# define Y_LUI_OP 354
# define Y_LWC0_OP 355
# define Y_LWC1_OP 356
# define Y_LWC2_OP 357
# define Y_LWC3_OP 358
# define Y_LWL_OP 359
# define Y_LWR_OP 360
# define Y_LW_OP 361
# define Y_MFC0_OP 362
# define Y_MFC1_OP 363
# define Y_MFC2_OP 364
# define Y_MFC3_OP 365
# define Y_MFHI_OP 366
# define Y_MFLO_OP 367
# define Y_MOV_D_OP 368
# define Y_MOV_S_OP 369
# define Y_MTC0_OP 370
# define Y_MTC1_OP 371
# define Y_MTC2_OP 372
# define Y_MTC3_OP 373
# define Y_MTHI_OP 374
# define Y_MTLO_OP 375
# define Y_MULTU_OP 376
# define Y_MULT_OP 377
# define Y_MUL_D_OP 378
# define Y_MUL_S_OP 379
# define Y_NEG_D_OP 380
# define Y_NEG_S_OP 381
# define Y_NOR_OP 382
# define Y_ORI_OP 383
# define Y_OR_OP 384
# define Y_PFW_OP 385
# define Y_RFE_OP 386
# define Y_SB_OP 387
# define Y_SH_OP 388
# define Y_SLLV_OP 389
# define Y_SLL_OP 390
# define Y_SLTIU_OP 391
# define Y_SLTI_OP 392
# define Y_SLTU_OP 393
# define Y_SLT_OP 394
# define Y_SRAV_OP 395
# define Y_SRA_OP 396
# define Y_SRLV_OP 397
# define Y_SRL_OP 398
# define Y_SUBU_OP 399
# define Y_SUB_D_OP 400
# define Y_SUB_OP 401
# define Y_SUB_S_OP 402
# define Y_SWC0_OP 403
# define Y_SWC1_OP 404
# define Y_SWC2_OP 405
# define Y_SWC3_OP 406
# define Y_SWL_OP 407
# define Y_SWR_OP 408
# define Y_SW_OP 409
# define Y_SYSCALL_OP 410
# define Y_TLBP_OP 411
# define Y_TLBR_OP 412
# define Y_TLBWI_OP 413
# define Y_TLBWR_OP 414
# define Y_XORI_OP 415
# define Y_XOR_OP 416
# define Y_ABS_POP 417
# define Y_BAL_POP 418
# define Y_BEQZ_POP 419
# define Y_BGEU_POP 420
# define Y_BGE_POP 421
# define Y_BGTU_POP 422
# define Y_BGT_POP 423
# define Y_BLEU_POP 424
# define Y_BLE_POP 425
# define Y_BLTU_POP 426
# define Y_BLT_POP 427
# define Y_BNEZ_POP 428
# define Y_B_POP 429
# define Y_LA_POP 430
# define Y_LD_POP 431
# define Y_LI_POP 432
# define Y_LI_D_POP 433
# define Y_LI_S_POP 434
# define Y_L_D_POP 435
# define Y_L_S_POP 436
# define Y_MFC1_D_POP 437
# define Y_MTC1_D_POP 438
# define Y_MOVE_POP 439
# define Y_MULOU_POP 440
# define Y_MULO_POP 441
# define Y_MUL_POP 442
# define Y_NEGU_POP 443
# define Y_NEG_POP 444
# define Y_NOP_POP 445
# define Y_NOT_POP 446
# define Y_REMU_POP 447
# define Y_REM_POP 448
# define Y_ROL_POP 449
# define Y_ROR_POP 450
# define Y_SD_POP 451
# define Y_SEQ_POP 452
# define Y_SGEU_POP 453
# define Y_SGE_POP 454
# define Y_SGTU_POP 455
# define Y_SGT_POP 456
# define Y_SLEU_POP 457
# define Y_SLE_POP 458
# define Y_SNE_POP 459
# define Y_S_D_POP 460
# define Y_S_S_POP 461
# define Y_ULHU_POP 462
# define Y_ULH_POP 463
# define Y_ULW_POP 464
# define Y_USH_POP 465
# define Y_USW_POP 466
# define Y_ALIAS_DIR 467
# define Y_ALIGN_DIR 468
# define Y_ASCII_DIR 469
# define Y_ASCIIZ_DIR 470
# define Y_ASM0_DIR 471
# define Y_BGNB_DIR 472
# define Y_BYTE_DIR 473
# define Y_COMM_DIR 474
# define Y_DATA_DIR 475
# define Y_DOUBLE_DIR 476
# define Y_ENDB_DIR 477
# define Y_ENDR_DIR 478
# define Y_END_DIR 479
# define Y_ENT_DIR 480
# define Y_ERR_DIR 481
# define Y_EXTERN_DIR 482
# define Y_FILE_DIR 483
# define Y_FLOAT_DIR 484
# define Y_FMASK_DIR 485
# define Y_FRAME_DIR 486
# define Y_GLOBAL_DIR 487
# define Y_HALF_DIR 488
# define Y_K_TEXT_DIR 489
# define Y_K_DATA_DIR 490
# define Y_LABEL_DIR 491
# define Y_LCOMM_DIR 492
# define Y_LIVEREG_DIR 493
# define Y_LOC_DIR 494
# define Y_MASK_DIR 495
# define Y_NOALIAS_DIR 496
# define Y_OPTIONS_DIR 497
# define Y_RDATA_DIR 498
# define Y_REPEAT_DIR 499
# define Y_SDATA_DIR 500
# define Y_SET_DIR 501
# define Y_SPACE_DIR 502
# define Y_STRUCT_DIR 503
# define Y_TEXT_DIR 504
# define Y_VERSTAMP_DIR 505
# define Y_VREG_DIR 506
# define Y_WORD_DIR 507
